////////////////////////////////////////////////////////////////////////////////////////////
// SX18/20/28/48/52 Virtual Peripheral Library
// 	all of the following are to be called from within timer ISR.
// 1) UART - VP_UART();
// 2) UART1 - VP_UART1(); - (relies on UARTfs/intPeriod defined in UART)
// 3) clock - VP_clock();
// 4) pwm0, pwm1, adc0, adc1 - VP_pwm_adc();
// pwm's and adc's all must reside in same thread and use same SX port - ie:RA/RB/RC
////////////////////////////////////////////////////////////////////////////////////////////
// Author(s): 	Marc Gebauer
//				Chris Fogelklou
//				ANAND.K.C
// Date 31 May 2005
////////////////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************************
 Uncomment one of the following to enable Virtual Peripherals
 *****************************************************************************************/
#define UART						// Enables VP: UART
#define UART1						// Enables VP: UART1
#define clock						// Enables VP: clock (milliseconds/HMS/days)
//#define pwm0						// Enables pwm0
//#define pwm1						// Enables pwm1
//#define adc0						// Enables adc0
//#define adc1						// Enables adc1

/*****************************************************************************************
 ----------------------------------VP Register definitions------------------------------
 *****************************************************************************************/
char flags=0;						// stores bit-wise operators like flags

#ifdef UART
/*****************************************************************************************
 UART - definitions

 --------------the Port Assignment in sx_config.h needs to be set.-------------
 example - for Tx=RA.3 / Rx=RA.2:
 
 #ifdef TXRX_invert
 #define RA_latch	00000000b		//SX18/20/28/48/52 port A latch init
 #else
 #define RA_latch	00001000b		//SX18/20/28/48/52 port A latch init
 #endif
 #define RA_DDIR	11110111b		//SX18/20/28/48/52 port A DDIR value
 #define RA_LVL		00000000b		//SX18/20/28/48/52 port A LVL value
 #define RA_PLP		00001111b		//SX18/20/28/48/52 port A PLP value
 
 *****************************************************************************************/

/*****************************************************************************************
 Uncomment the following to set the UART ISR divider.
 *****************************************************************************************/
//#define UARTisrDiv 1	// this one if running UART directly inside timer Interrupt.			

//#define UARTisrDiv 2	// this one if running UART inside Multitasker thread1.			
						// Max baud for UARTisrDiv 2 is 38400.
#define UARTisrDiv 4	// this one if running UART inside Multitasker thread2.			
						// Max baud for UARTisrDiv 2 is 38400.
//#define UARTisrDiv 16	// this one if running UART inside Multitasker thread(3-8).
						// Max baud for UARTisrDiv 16 is 4800.
			
/*****************************************************************************************
 Set the following to setup the uart vp port assignments
 *****************************************************************************************/
#define Tx RC.7			// comment out, to disable transmit routines
//#define Rx RA.2			// comment out, to disable receive routines
//#define TxOE RA.1		// Transmit output enable for MAX1482

/*****************************************************************************************
 Uncomment the following to use SX chip without inverting buffer such as MAX232
 *****************************************************************************************/
//#define tristate		// Enable UART Tx tristate mode via MAX1482.

/*****************************************************************************************
 Uncomment one of the following to run the uart vp at the required baud rate
 * 31250 baud is exclusive to intPeriod. If used UART1 must also be set for 31250.
 *****************************************************************************************/
//#define baud1200		// baud rate of 1.2 Kbps
//#define baud2400		// baud rate of 2.4 Kbps
//#define baud4800		// baud rate of 4.8 Kbps
//#define baud9600		// baud rate of 9.6 kbps
#define baud1920		// baud rate of 19.2kbps
//#define baud3125		// baud rate of 31.25kbps
//#define baud3840		// baud rate of 38.4kbps
//#define baud5760		// baud rate of 57.6kbps 

/*****************************************************************************************
 Uncomment the following to enable UART parity - default is even parity.
 *****************************************************************************************/
//#define Parity
/*****************************************************************************************
 Uncomment the following to change parity to odd.
 *****************************************************************************************/
//#define oddParity

/*****************************************************************************************
 Uncomment the following to use SX chip without inverting buffer such as MAX232 or MAX1482
 *****************************************************************************************/
//#define TXRX_invert		// Invert Transmit and Recieve bits for direct couple to SX chip.
						// Make sure to add a 4.7K resistor to Rx in series.

/*****************************************************************************************
 	UART Constants values
 *****************************************************************************************/
	#ifdef baud1200
	#define UARTBaud 1200
	#endif
	
	#ifdef baud2400
	#define UARTBaud 2400
	#endif
	
	#ifdef baud4800
	#define UARTBaud 4800
	#endif
	
	#ifdef baud9600
	#define UARTBaud 9600
	#endif
	
	#ifdef baud1920
	#define UARTBaud 19200
	#endif

	#ifdef baud3125
	#define UARTBaud 31250
	#endif

	#ifdef baud3840
	#define UARTBaud 38400
	#endif

	#ifdef baud5760
	#define UARTBaud 57600
	#endif

	#ifdef baud3125

	#define intPeriod 203
	#define UARTfs 250000

	#else

	#define intPeriod 220
	#define UARTfs 230400

	#endif

#define UARTDivide (UARTfs/(UARTBaud*UARTisrDiv))
#define UARTtDelay (UARTDivide/2)*3+1


/*********************************************************************************************
 -------------------------------- UART Register definitions----------------------------
 *********************************************************************************************/
	#ifdef Tx
char Txhigh;			//hi byte to transmit
char Txlow;				//low byte to transmit
char Txcount;			//number of bits sent
char Txdivide;			//xmit timing counter
char TxParBitSum;		//sum of 1s transmit. bit 0 holds parity
	#endif

	#ifdef Rx
/*********************************************************************************************
	RxParBit		=	flags.RxParBit	; Parity of Receive byte
	RxRdy			=	flags.RxRdy		; indicates the reception of a bit from the UART
 *********************************************************************************************/
#define RxParBit 0
#define RxRdy 1

char Rxcount;			//number of bits received
char Rxdivide;			//receive timing counter
char Rxbyte=0;			//buffer for incoming byte
	#endif

/*--------------------------------------------VP: UART -----------------------------------

;*****************************************************************************************
; Virtual Peripheral: Universal Asynchronous Receiver Transmitter (UART)
; - run inside timer interrupt.
; These routines send and receive  serial data, and are currently
; configured (though modifications can be made) for the popular
; "No parity-checking, 8 data bit, 1 stop bit" (N,8,1) or
; "TRANSMIT parity with no RECEIVE checking, 8 data bits, 1stop bit" ((E/O),8,1) data formats.
; (parity handled by sendbyte routine. 'receive byte' is user responsiblity)
;
; RECEIVING: The flags.RxRdy is set high whenever a valid byte of data has been
; received and it is the calling routine's responsibility to reset this flag
; once the incoming data has been collected.
;****************************************************************************************
; TRANSMITTING: The transmit routine requires the data to be inverted
; and loaded (Txhigh+Txlow) register pair (with the inverted 8 data bits
; stored in Txhigh and Txlow bit 7 set high to act as a start bit). 
; Then the number of bits ready for transmission (11=1 start + 8 data + 2 stop or 1 parity and 1 stop).
; must be loaded into the Txcount register. As soon as this latter is done,
; the transmit routine immediately begins sending the data.
; This routine has a varying execution rate and therefore should always be
; placed after any timing-critical virtual peripherals such as timers,
; adcs, pwms, etc.
; Note: The transmit and receive routines are independent and either may be
;	removed, if not needed, to reduce execution time and memory usage.
;       Input variable(s) : Txlow (only high bit used), Txhigh, Txcount
;       Output variable(s) : none
;       Variable(s) affected : Txdivide, TxParBitSum
;	Program cycles: ?? worst case
;	Variable Length?  Yes.
;****************************************************************************************
; RECEIVING: The flags.RxRdy is set high whenever a valid byte of data has been
; received and it is the calling routine's responsibility to reset this flag
; once the incoming data has been collected.
;       Output variable(s) : flags.RxRdy, rxParBit, rx_byte
;       Variable(s) affected : rx_divide, rx_count
;       Flag(s) affected : flags.RxRdy, flags.RxParBit
;	Program cycles: ?? worst case.
;	Variable Length?  Yes.
;*****************************************************************************************/

void VP_UART(void)
{
	#ifdef Tx
Transmit:
		asm bank	_Txdivide		;2
		asm decsz	_Txdivide		;1 only execute the transmit routine
		asm jmp		_TxOut			;1 
		asm mov		w,#UARTDivide	;1 load UART baud rate (50MHz)
		asm bank	_Txdivide		;2
		asm mov		_Txdivide,w		;1  
		asm bank	_Txcount		;2
		asm test    _Txcount		;1 are we sending?
		asm sb		$3.2			;1
		asm	jmp		_txbit			;1
		#ifdef tristate
		asm clrb	TxOE			; no, set rs422 output to high impedence
		#endif
		asm jmp		_TxOut;			;1
		
txbit:
		#ifdef	tristate
		asm setb	TxOE			;1 yes, set rs422 output to low impedence
		#endif
		#ifdef Parity
		asm setb	$03.0			;1 yes, enable parity
		#else
		asm clrb 	$03.0			;1 yes, ready stop bit and disable parity
		#endif
		asm bank	_Txhigh			;2
		asm rr      _Txhigh			;1 and shift to next bit
		asm bank	_Txlow			;2
		asm rr      _Txlow			;1  
		asm bank	_Txcount		;2
		asm dec     _Txcount		;1 decrement bit counter
		asm bank	_Txlow			;2
		asm sb		_Txlow.6		;1 is bit set? if not,
		asm jmp		_TxbitOut		;1 exit (a clr bit on Txcount 10 will disable parity)
		if(Txcount < 10) ++TxParBitSum;	//4 if so, and if a data bit, adjust parity bit sum
		if(Txcount == 1){				//4 if data bits done, output parity bit
			asm bank	_TxParBitSum	;2
		#ifdef oddParity
			++TxParBitSum;			//1 make odd parity
		#endif
			asm	rr		_TxParBitSum	;1
			asm bank	_Txlow		;2
			asm rr		_Txlow		;1 ready parity or stop bit
			asm clrb 	$03.0		;1 ready stop bit
			asm rr		_Txlow		;1
		}
TxbitOut:
		#ifdef TXRX_invert
		asm bank	_Txlow			;2
		asm snb		_Txlow.6		;1 output next bit
		asm setb	Tx				;1  
		asm sb		_Txlow.6		;1  
		asm clrb	Tx				;1,??  
		#else
		asm bank	_Txlow			;2
		asm snb		_Txlow.6		;1 output next bit
		asm clrb	Tx				;1  
		asm sb		_Txlow.6		;1  
		asm setb	Tx				;1,??  
		#endif
TxOut:
	#endif

	#ifdef Rx
Receive:
		#ifdef TXRX_invert
		asm snb		Rx				;1 get current Rx bit
		asm clrb 	$03.0			;1  
		asm sb		Rx				;1  
		asm setb 	$03.0			;1  
		#else
		asm sb		Rx				;1 get current Rx bit
		asm clrb 	$03.0			;1					  
		asm snb		Rx				;1  
		asm setb 	$03.0			;1  
		#endif
		asm bank	_Rxcount		;2
		asm test    _Rxcount		;1 currently receiving byte?
		asm sb 		$03.2			;1  
		asm jmp		_rxbit			;1 if so, jump ahead
		asm mov     w,#9			;1 in case start, ready 9 bits
		asm bank	_Rxcount		;2
		asm sb		$03.0			;1 skip ahead if not start bit
		asm mov     _Rxcount,w		;1 it is, so renew bit count
		asm mov     w,#UARTtDelay	;1 ready 1.5 bit periods (50MHz)		
		asm bank	_Rxdivide		;2
		asm mov     _Rxdivide,w		;1  
rxbit:
		asm bank	_Rxdivide		;2
		asm decsz	_Rxdivide		;1 middle of next bit?
        asm jmp		_RxOut			;1 
		asm mov		w,#UARTDivide	;1 yes, ready 1 bit period (50MHz)
		asm bank	_Rxdivide		;2
		asm mov		_Rxdivide,w		;1  
		asm bank	_Rxcount		;2
		asm dec     _Rxcount		;1
		asm bank	_flags			;2
		asm snb		_flags.RxParBit	;1 is this the parity bit? if so,
		asm jmp		_rxParbit1		;1 jump ahead
		asm bank	_Rxbyte			;2 else
		asm snb 	$03.2			;1 any more data bits? if not,
		asm jmp		_rxParbit		;1 jump ahead
		asm rr      _Rxbyte			;1 else save bit
		asm jmp		_RxOut			;1 exit
rxParbit:
		#ifdef Parity
		asm bank	_Rxcount		;2
		asm inc 	_Rxcount		;1 prep for parity
		asm bank	_flags			;2
		asm setb 	_flags.RxParBit	;1 declare next bit is parity, set flag
		asm	jmp		_RxOut			;1
		#endif
rxParbit1:
		asm bank	_flags			;2
		asm clrb	_flags.RxParBit ;1 reset for next byte
		asm setb 	_flags.RxRdy	;1,?? then set byte received flag
RxOut:
	#endif
}
/*****************************************************************************************
 end - VP_UART()
 *****************************************************************************************/

/*****************************************************************************************
; 	Function	: sendbyte
;	INPUTS		: param00_Sendbyte
; 	OUTPUTS		: Outputs TxHigh,TxLow, TcCount
; 	Send byte via UART
;*****************************************************************************************/
	#ifdef Tx
void sendbyte(char TxByte)
{
		char tmp;

		tmp=TxByte;						// also moves content into 'w' register
		asm bank    _Txcount
wait:   
		asm test    _Txcount			;  wait for not busy
		asm sb 		$03.2
		goto wait;
		asm xor 	w,#$FF				;  ready bits (inverse logic)
		asm bank	_Txhigh
		asm mov		_Txhigh,w			;  store data byte
		asm bank	_Txlow
		asm setb	_Txlow.7			;  set up start bit
		asm mov		w,#11				; 1 start + 8 data + (parity and stop or 2 stop bits)
		asm bank	_Txcount
		asm mov     _Txcount,w
		TxParBitSum=0;					//prep for using parity.
}
/*****************************************************************************************
 end - sendbyte()
 *****************************************************************************************/

/*****************************************************************************************
; 	Function	: Send string
;	INPUTS		: param00_printf
; 	OUTPUTS		: Output param00_Sendbyte
; 	Send byte via sendbyte to UART
;*****************************************************************************************/

void printf( const char* text )
{
    char i = 0;
    while( text[i] != 0 )
        sendbyte( text[i++] ); 
		
}
	#endif
#endif
/*****************************************************************************************
 end - printf()
 *****************************************************************************************/


#ifdef UART1
/*****************************************************************************************
 UART1 - definitions

 --------------the Port Assignment in sx_config.h needs to be set.-------------
 example - for Tx1=RB.6 / Rx1=RB.5:
 
 #ifdef U1TXRX_invert
 #define RB_latch	00000000b		//SX18/20/28/48/52 port B latch init;intial value after reset
 #else
 #define RB_latch	01000000b		//SX18/20/28/48/52 port B latch init;intial value after reset
 #endif
 #define RB_DDIR	10111110b		//SX18/20/28/48/52 port B DDIR value; 0=Output,1=Input
 #define RB_ST		11111111b		//SX18/20/28/48/52 port B ST value; 0=Enable,1=Disable
 #define RB_LVL		00000000b		//SX18/20/28/48/52 port B LVL value; 0=CMOS,1=TTL
 #define RB_PLP		01100000b		//SX18/20/28/48/52 port B PLP value; 0=Enable,1=Disable
 
 *****************************************************************************************/

/*****************************************************************************************
 Uncomment the following to set the UART1 ISR divider.
 *****************************************************************************************/
//#define UART1isrDiv 1	// this one if running UART1 directly inside timer Interrupt.			
#define UART1isrDiv 2	// this one if running UART1 inside Multitasker thread1.			
//#define UART1isrDiv 4	// this one if running UART1 inside Multitasker thread1.
//#define UART1isrDiv 16	// this one if running UART1 inside Multitasker thread(3-8).
						// Max baud for UART1isrDiv 16 is 4800.
			
/*****************************************************************************************
 Set the following to set the uart vp port assignments
 *****************************************************************************************/
#define Tx1 RA.3			// comment out, to disable transmit routines
#define Rx1 RA.2			// comment out, to disable receive routines

/*****************************************************************************************
 Uncomment one of the following to run the uart vp at the required baud rate
 * 31250 baud is exclusive to intPeriod. If used UART must also be set for 31250.
 *****************************************************************************************/
//#define U1baud1200		// baud rate of 1.2 Kbps
//#define U1baud2400		// baud rate of 2.4 Kbps
//#define U1baud4800		// baud rate of 4.8 Kbps
//#define U1baud9600		// baud rate of 9.6 kbps
//#define U1baud1920		// baud rate of 19.2kbps
//#define U1baud3125		// baud rate of 31.25kbps
#define U1baud3840		// baud rate of 38.4kbps
//#define U1baud5760		// baud rate of 57.6kbps 

/*****************************************************************************************
 Uncomment the following to enable UART1 parity - default is even parity.
 *****************************************************************************************/
//#define U1Parity
/*****************************************************************************************
 Uncomment the following to change parity to odd.
 *****************************************************************************************/
//#define U1oddParity

/*****************************************************************************************
 Uncomment the following to use SX chip without inverting buffer such as MAX232
 *****************************************************************************************/
//#define U1TXRX_invert		// Invert Transmit and Recieve bits for direct couple to SX chip.
						// Make sure to add a 4.7K resistor to Rx1 in series.

/*****************************************************************************************
 	UART1 Constants values
 *****************************************************************************************/
	#ifdef U1baud1200
	#define UART1Baud 1200
	#endif
	
	#ifdef U1baud2400
	#define UART1Baud 2400
	#endif
	
	#ifdef U1baud4800
	#define UART1Baud 4800
	#endif
	
	#ifdef U1baud9600
	#define UART1Baud 9600
	#endif
	
	#ifdef U1baud1920
	#define UART1Baud 19200
	#endif

	#ifdef U1baud3125
	#define UART1Baud 31250
	#endif

	#ifdef U1baud3840
	#define UART1Baud 38400
	#endif

	#ifdef U1baud5760
	#define UART1Baud 57600
	#endif

#define UART1Divide (UARTfs/(UART1Baud*UART1isrDiv))
#define UART1tDelay (UART1Divide/2)*3+1


/*********************************************************************************************
 -------------------------------- UART1 Register definitions----------------------------
 *********************************************************************************************/
	#ifdef Tx1
char Tx1high;			//hi byte to transmit
char Tx1low;			//low byte to transmit
char Tx1count;			//number of bits sent
char Tx1divide;			//xmit timing counter
char Tx1ParBitSum;		//sum of 1s transmit. bit 0 holds parity
	#endif

	#ifdef Rx1
/*********************************************************************************************
	Rx1ParBit		=	flags.Rx1ParBit	; Parity of Receive byte
	Rx1Rdy			=	flags.Rx1Rdy	; indicates the reception of a bit from the UART1
 *********************************************************************************************/
#define Rx1ParBit 2
#define Rx1Rdy 3

char Rx1count;			//number of bits received
char Rx1divide;			//receive timing counter
char Rx1byte=0;			//buffer for incoming byte
	#endif

/*--------------------------------------------VP: UART1 -----------------------------------

;*****************************************************************************************
; Virtual Peripheral: Universal Asynchronous Receiver Transmitter (UART1)
; - run inside timer interrupt.
; These routines send and receive  serial data, and are currently
; configured (though modifications can be made) for the popular
; "No parity-checking, 8 data bit, 1 stop bit" (N,8,1) or
; "TRANSMIT parity with no RECEIVE checking, 8 data bits, 1stop bit" ((E/O),8,1) data formats.
; (parity handled by sendbyte routine. 'receive byte' is user responsiblity)
;
; RECEIVING: The flags.Rx1Rdy is set high whenever a valid byte of data has been
; received and it is the calling routine's responsibility to reset this flag
; once the incoming data has been collected.
;****************************************************************************************
; TRANSMITTING: The transmit routine requires the data to be inverted
; and loaded (Tx1high+Tx1low) register pair (with the inverted 8 data bits
; stored in Tx1high and Tx1low bit 7 set high to act as a start bit). 
; Then the number of bits ready for transmission (11=1 start + 8 data + 2 stop or 1 parity and 1 stop).
; must be loaded into the Tx1count register. As soon as this latter is done,
; the transmit routine immediately begins sending the data.
; This routine has a varying execution rate and therefore should always be
; placed after any timing-critical virtual peripherals such as timers,
; adcs, pwms, etc.
; Note: The transmit and receive routines are independent and either may be
;	removed, if not needed, to reduce execution time and memory usage.
;       Input variable(s) : Tx1low (only high bit used), Tx1high, Tx1count
;       Output variable(s) : none
;       Variable(s) affected : Tx1divide, Tx1ParBitSum
;	Program cycles: ?? worst case
;	Variable Length?  Yes.
;****************************************************************************************
; RECEIVING: The flags.Rx1Rdy is set high whenever a valid byte of data has been
; received and it is the calling routine's responsibility to reset this flag
; once the incoming data has been collected.
;       Output variable(s) : flags.Rx1Rdy, Rx1ParBit, Rx1byte
;       Variable(s) affected : Rx1divide, Rx1count
;       Flag(s) affected : flags.Rx1Rdy, flags.Rx1ParBit
;	Program cycles: ?? worst case.
;	Variable Length?  Yes.
;*****************************************************************************************/

void VP_UART1(void)
{
	#ifdef Tx1
U1Transmit:
		asm bank	_Tx1divide		;2
		asm decsz	_Tx1divide		;1 only execute the transmit routine
		asm jmp		_Tx1Out			;1 
		asm mov		w,#UART1Divide	;1 load UART1 baud rate (50MHz)
		asm bank	_Tx1divide		;2
		asm mov		_Tx1divide,w	;1  
		asm bank	_Tx1count		;2
		asm test    _Tx1count		;1 are we sending?
		asm snb 	$03.2			;1
		asm jmp		_Tx1Out;		;1
Tx1bit:
		#ifdef U1Parity
		asm setb	$03.0			;1 yes, enable parity
		#else
		asm clrb 	$03.0			;1 yes, ready stop bit and disable parity
		#endif
		asm bank	_Tx1high			;2
		asm rr      _Tx1high			;1 and shift to next bit
		asm bank	_Tx1low			;2
		asm rr      _Tx1low			;1  
		asm bank	_Tx1count		;2
		asm dec     _Tx1count		;1 decrement bit counter
		asm bank	_Tx1low			;2
		asm sb		_Tx1low.6		;1 is bit set? if not,
		asm jmp		_Tx1bitOut		;1 exit (a clr bit on Tx1count 10 will disable parity)
		if(Tx1count < 10) ++Tx1ParBitSum;	//4 if so, and if a data bit, adjust parity bit sum
		if(Tx1count == 1){			//4 if data bits done, output parity bit
			asm bank	_Tx1ParBitSum	;2
		#ifdef U1oddParity
			++Tx1ParBitSum;			//1 make odd parity
		#endif
			asm	rr		_Tx1ParBitSum	;1
			asm bank	_Tx1low		;2
			asm rr		_Tx1low		;1 ready parity or stop bit
			asm clrb 	$03.0		;1 ready stop bit
			asm rr		_Tx1low		;1
		}
Tx1bitOut:
		#ifdef U1TXRX_invert
		asm bank	_Tx1low			;2
		asm snb		_Tx1low.6		;1 output next bit
		asm setb	Tx1				;1  
		asm sb		_Tx1low.6		;1  
		asm clrb	Tx1				;1,??  
		#else
		asm bank	_Tx1low			;2
		asm snb		_Tx1low.6		;1 output next bit
		asm clrb	Tx1				;1  
		asm sb		_Tx1low.6		;1  
		asm setb	Tx1				;1,??  
		#endif
Tx1Out:
	#endif

	#ifdef Rx1
U1Receive:
		#ifdef U1TXRX_invert
		asm snb		Rx1				;1 get current Rx1 bit
		asm clrb 	$03.0			;1  
		asm sb		Rx1				;1  
		asm setb 	$03.0			;1  
		#else
		asm sb		Rx1				;1 get current Rx1 bit
		asm clrb 	$03.0			;1					  
		asm snb		Rx1				;1  
		asm setb 	$03.0			;1  
		#endif
		asm bank	_Rx1count		;2
		asm test    _Rx1count		;1 currently receiving byte?
		asm sb 		$03.2			;1  
		asm jmp		_Rx1bit			;1 if so, jump ahead
		asm mov     w,#9			;1 in case start, ready 9 bits
		asm bank	_Rx1count		;2
		asm sb		$03.0			;1 skip ahead if not start bit
		asm mov     _Rx1count,w		;1 it is, so renew bit count
		asm mov     w,#UART1tDelay	;1 ready 1.5 bit periods (50MHz)		
		asm bank	_Rx1divide		;2
		asm mov     _Rx1divide,w	;1  
Rx1bit:
		asm bank	_Rx1divide		;2
		asm decsz	_Rx1divide		;1 middle of next bit?
        asm jmp		_Rx1Out			;1 
		asm mov		w,#UART1Divide	;1 yes, ready 1 bit period (50MHz)
		asm bank	_Rx1divide		;2
		asm mov		_Rx1divide,w	;1  
		asm bank	_Rx1count		;2
		asm dec     _Rx1count		;1
		asm bank	_flags			;2
		asm snb		_flags.Rx1ParBit	;1 is this the parity bit? if so,
		asm jmp		_Rx1Parbit1		;1 jump ahead
		asm bank	_Rx1byte		;2 else
		asm snb 	$03.2			;1 any more data bits? if not,
		asm jmp		_Rx1Parbit		;1 jump ahead
		asm rr      _Rx1byte		;1 else save bit
		asm jmp		_Rx1Out			;1 exit
Rx1Parbit:
		#ifdef U1Parity
		asm bank	_Rx1count		;2
		asm inc 	_Rx1count		;1 prep for parity
		asm bank	_flags			;2
		asm setb 	_flags.Rx1ParBit	;1 declare next bit is parity, set flag
		asm	jmp		_Rx1Out			;1
		#endif
Rx1Parbit1:
		asm bank	_flags			;2
		asm clrb	_flags.Rx1ParBit	;1 reset for next byte
		asm setb 	_flags.Rx1Rdy	;1,?? then set byte received flag
Rx1Out:
	#endif
}
/*****************************************************************************************
 end - VP_UART()
 *****************************************************************************************/

/*****************************************************************************************
; 	Function	: sendbyte
;	INPUTS		: param00_Sendbyte
; 	OUTPUTS		: Outputs Tx1High,Tx1Low, Tx1Count
; 	Send byte via UART1
;*****************************************************************************************/
	#ifdef Tx1
void sendbyte1(char Tx1Byte)
{
		char tmp;

		tmp=Tx1Byte;					// also moves content into 'w' register
		asm bank    _Tx1count
wait1:   
		asm test    _Tx1count			;  wait for not busy
		asm sb 		$03.2
		goto wait1;
		asm xor 	w,#$FF				;  ready bits (inverse logic)
		asm bank	_Tx1high
		asm mov		_Tx1high,w			;  store data byte
		asm bank	_Tx1low
		asm setb	_Tx1low.7			;  set up start bit
		asm mov		w,#11				; 1 start + 8 data + (parity and stop or 2 stop bits)
		asm bank	_Tx1count
		asm mov     _Tx1count,w
		Tx1ParBitSum=0;					//prep for using parity.
}
/*****************************************************************************************
 end - sendbyte1()
 *****************************************************************************************/

/*****************************************************************************************
; 	Function	: Send string
;	INPUTS		: param00_printf1
; 	OUTPUTS		: Output param00_Sendbyte
; 	Send byte via sendbyte to UART1
;*****************************************************************************************/

void printf1( const char* text )
{
    char i = 0;
    while( text[i] != 0 )
        sendbyte1( text[i++] ); 
}
	#endif
#endif
/*****************************************************************************************
 end - printf1()
 *****************************************************************************************/


#ifdef clock
/*****************************************************************************************
 clock - definitions
 *****************************************************************************************
 Uncomment the following to set the ISR divider.
 *****************************************************************************************/
//#define clockisrDiv 1		// this one if running clock directly inside timer interrupt.			
//#define clockisrDiv 2		// this one if running clock inside Multitasker thread1.			
#define clockisrDiv 16	// this one if running clock inside Multitasker thread(2-8).

/*****************************************************************************************
 Uncomment the following to Time Clock mode of operation 
 *****************************************************************************************/
#define clock_hms			// include sec, min, hours
#define clock_days		// include day counter

/*********************************************************************************************
 -------------------------------- UART Register definitions----------------------------
 *********************************************************************************************/
char time_base_correction=0;					//used to make accurate
char time_base_lo=0;							//time base delay (low byte)
char time_base_hi=0;							//time base delay (high byte)
short milliseconds=0;							//millisecond count
short ms_timer=0;								//millisecond count down timer
#ifdef clock_hms
char seconds=0;									//seconds count
char sec_timer=0;								//second count down timer
char minutes=0;									//minutes count
char hours=0;									//hours count
#ifdef clock_days
char days=0;									//days count
#endif				
#endif				

/*****************************************************************************************
; Virtual Peripheral: Time Clock - run inside timer interrupt.
; This routine maintains a real-time clock count (in msec) and allows processing
; of routines which only need to be run once every millisecond.
;
;	Input variable(s): time_base_lo,time_base_hi,msec_lo,msec_hi
;				seconds, minutes, hours, days
;	Output variable(s): millisecond
;				seconds, minutes, hours, days
;	Variable(s) affected: time_base_lo,time_base_hi,msec_lo,msec_hi
;				seconds, minutes, hours, days
;	Flag(s) affected: 
;
;	Size : 17/39/45 bytes (depending upon clock type)
;		+ 1 if bank select needed
;	Timing (turbo) : [99.9% of time] 14 cycles
;			 [0.1% of time] 17/39/45 cycles (or less)
;			+ 1 if bank select needed
;*****************************************************************************************/

void VP_clock(void)
{
// bandaid I'm trying to make clock more accurate
		++time_base_correction;				//incement used to toggle first bit
		time_base_correction=time_base_correction & 1;	// mask out bits 2 thru 8
		asm mov		w,#intPeriod+_time_base_correction	;load period between interrupts w correction
// end bandaid

//		asm mov		w,#intPeriod
		asm bank    _time_base_lo
		asm add		_time_base_lo,w			;add it to time base 
		asm bank    _time_base_hi
		asm snb 	$03.0					;skip ahead if no underflow
		asm inc		_time_base_hi			;yes overflow, adjust high byte
		asm mov		w,#(195/clockisrDiv)	;check for 1 msec click
		asm bank    _time_base_hi
		asm mov		w,_time_base_hi-w		;Is high byte above or equal?
		asm mov		w,#(50000/clockisrDiv)	;load instr. count low byte
		asm bank    _time_base_lo
		asm snb 	$03.2					;If hi byte equal, skip ahead
		asm mov		w,_time_base_lo-w		;check low byte vs. time base
		asm sb		$03.0					;skip ahead if low
		asm jmp		_done_clock				;If not, end clock routine
got_tick:
// millisecond countdown timer service routine here
		if(ms_timer == 0) goto ms_cntdwn_done;	// check if count = zero, if not
		--ms_timer; 						//decrement count
ms_cntdwn_done:
// end millisecond countdown timer service routine
	asm bank    _time_base_hi
		asm clr		_time_base_hi			;Yes, adjust time_base reg.'s
		asm mov 	w,#(50000/clockisrDiv)	;
		asm bank    _time_base_lo
		asm sub		_time_base_lo,w			; leaving time remainder
		++milliseconds;						//adjust millisecond count
		#ifdef		clock_hms				//do we want sec, min, hours?
		if(milliseconds<1000) goto done_clock;		//check for 1000 msec (1 sec tick)
		milliseconds=0;						//clear millisecond count
got_second:
// second countdown timer service routine here
		asm bank	_sec_timer
		asm	test	_sec_timer				; check if count = zero
		asm	sb		$3.2					; if not,
		asm dec		_sec_timer				; decrement count	
// end second countdown timer service routine
		asm bank    _seconds
		asm inc		_seconds				;increment seconds count
		asm mov		w,#60					;60 seconds per minute
		asm bank    _seconds
		asm mov		w,_seconds-w			;are we at minute tick yet
		asm sb		$03.2
		asm jmp		_done_clock				;if not, jump
got_minute:
		asm bank    _minutes
		asm inc		_minutes				;increment minutes count
		asm bank    _seconds
		asm clr		_seconds				;clear seconds count
		asm mov		w,#60					;60 minutes/hour
		asm bank    _minutes
		asm mov		w,_minutes-w			;are we at hour tick yet?
		asm sb		$03.2
		asm jmp		_done_clock				;if not, jump
got_hour:
		asm bank    _hours
		asm inc		_hours					;increment hours count
		asm bank    _minutes
		asm clr		_minutes				;clear minutes count

		#ifdef		clock_days				//do we want to count days?

		asm mov		w,#24					;24 hours per day
		asm bank    _hours
		asm mov		w,_hours-w				;are we at midnight?
		asm sb 		$03.2
		asm jmp		_done_clock				;if not, jump
got_day:
		asm bank    _days
		asm inc		_days					;increment days count
		asm bank    _hours
		asm clr		_hours					;clear hours count

		#endif								//<if> we wanted day count
		#endif								//<if> we wanted sec, min, hours
done_clock:
}
#endif
/*****************************************************************************************
 end - VP_clock()
 *****************************************************************************************/


/*****************************************************************************************
 PWM0,PWM1 / ADC0,ADC1 - definitions

 --------------the Port Assignment in sx_config.h needs to be set.-------------

 Example for default port assignments:
 #define RC_latch	00000000b		//SX18/20/28/48/52 port C latch init;intial value affter reset
 #define RC_DDIR	10101010b		//SX18/20/28/48/52 port C DDIR value; 0=Output,1=Input
 #define RC_ST		11111111b		//SX18/20/28/48/52 port C ST value; 0=Enable,1=Disable
 #define RC_LVL		00000000b		//SX18/20/28/48/52 port C LVL value; 0=CMOS,1=TTL
 #define RC_PLP		11111111b		//SX18/20/28/48/52 port C PLP value; 0=Enable,1=Disable

 *****************************************************************************************/

#define pa_Port rc							// SX PWM/ADC port 
#define pwm0_pin 0							// Pulse width mod. PWM0 output
#define pwm1_pin 2							// Pulse width mod. PWM1 output
#define adc0_out_pin 4						// ADC0 input pin
#define adc0_in_pin 5						// ADC0 output/calibrate pin
#define adc1_out_pin 6						// ADC1 input pin
#define adc1_in_pin 7						// ADC1 output/calibrate pin

#ifdef pwm0
#define PWM_ADC
#define pwm
#endif

#ifdef pwm1
#define PWM_ADC
#define pwm
#endif

#ifdef adc0
#define PWM_ADC
#define adc
#endif

#ifdef adc1
#define PWM_ADC
#define adc
#endif

#ifdef PWM_ADC
char port_buff;									// buffer - used by all
#endif

#ifdef pwm0
char pwm0out;									// pwm0out - value
char pwm0_acc;									//      - accumulator
#endif

#ifdef pwm1
char pwm1out;									// pwm1out - value
char pwm1_acc;									//      - accumulator
#endif

#ifdef adc0
char adc0in;									// adc0in - value
char adc0_count;								//      - real-time count
char adc0_acc;									//      - accumulator
#endif

#ifdef adc1
char adc1in;									// adc1in - value
char adc1_count;								//      - real-time count
char adc1_acc;									//      - accumulator
#endif


void VP_pwm_adc(void)
{
/*****************************************************************************************

 ***** Virtual Peripheral: Pulse Width Modulators

 These routines create an 8-bit programmable duty cycle output at the
 respective pwm port output pins whose duty cycle is directly proportional
 to the value in the corresponding pwm register. This value is added to an
 accumulator on each interrupt pass interrupt. When the addition causes a
 carry overflow, the ouput is set to the high part of its duty cycle.
 These routines are timing critical and must be placed before any
 variable-execution-rate code (like the UART, for example).

		Input variable(s) : pwm0out,pwm0_acc,pwm1out,pwm1_acc
		Output variable(s) : pwm port pins
		Variable(s) affected : 4, pwm0_acc, pwm1_acc
		Flag(s) affected : none
		Size : 2 bytes + 4 bytes (per pwm)
			+ 2 bytes shared with adc code (see below)
		Timing (turbo) : 2 cycles + 4 cycles (per pwm)
			+ 2 cycles shared with adc code (see below)

 *****************************************************************************************/
#ifdef pwm
		asm bank	_port_buff
		asm clr     _port_buff				;clear pwm/adc port buffer
#endif

#ifdef pwm0
		asm bank	_pwm0out	
		asm mov		w,_pwm0out
		asm bank	_pwm0_acc
		asm add     _pwm0_acc,w				;adjust pwm0out accumulator
		asm sb 		$03.0					;did it trigger?
		asm jmp		_pwm0_done				;no, exit
		asm bank	_port_buff
		asm setb    _port_buff.pwm0_pin		;else, toggle pwm0out high
pwm0_done:

#endif

#ifdef pwm1
		asm bank	_pwm1out	
		asm mov		w,_pwm1out
		asm bank	_pwm1_acc
		asm add     _pwm1_acc,w				;adjust pwm1out accumulator
		asm sb 		$03.0					;did it trigger?
		asm jmp		_pwm1_done				;no, exit
		asm bank	_port_buff
		asm setb    _port_buff.pwm1_pin		;else, toggle pwm1out high
pwm1_done:

#endif

#ifndef adc

update_RC:
		asm bank	_port_buff
		asm mov		w,_port_buff
		asm mov     pa_Port,w				;update cap. discharge pins
#endif

/*****************************************************************************************

 ***** Virtual Peripheral: Bitstream Analog to Digital Converters

 These routines allow an 8-bit value to be calculated which corresponds
 directly (within noise variation limits) with the voltage (0-5V) present
 at the respective adc port input pins. These routines are timing critical
 and must be placed before any variable-execution-rate code (like the UART,
 for example). The currentroutine has been optimized for a fixed execution rate.
 
		Input variable(s) : adc0in,adc0_acc,adc0_count,adc1in,adc1_acc,adc1_count 
		Output variable(s) : pwm port pins 
		Variable(s) affected : port_buff, pwm0_acc, pwm1_acc 
		Flag(s) affected : none 
		Size (version A) : 9 bytes + 7 bytes (per pwm) 
			+ 2 bytes shared with adc code (see below) 
		Size (version B) : 6 bytes + 10 bytes (per pwm) 
			+ 2 bytes shared with pwm code (see below) 
		Timing (turbo) 
		version A : 2 cycles shared with pwm code (see below) + 
			(a) [>99% of time] 11 cycles + 4 cycles (per adc) 
			(b) [<1% of time] 9 cycles + 7 cycles (per adc) 
		version B : 6 cycles + 10 cycles (per adc) 
			+ 2 cycles shared with pwm code (see below) 

 *****************************************************************************************/

#ifndef pwm
		asm bank	_port_buff
		asm clr		_port_buff				;clear pwm/adc port buffer 
#endif

#ifdef adc
char isrTmp=0;
		asm mov		w,>>pa_Port				;get current status of adc's 
		asm not		w						;complement inputs to outputs
		asm bank	_isrTmp_VP_pwm_adc
		asm clr		_isrTmp_VP_pwm_adc				;prep mask
		asm setb	_isrTmp_VP_pwm_adc.adc0_out_pin	;	"
		asm setb	_isrTmp_VP_pwm_adc.adc1_out_pin	;	"
		asm and     w,#_isrTmp_VP_pwm_adc	;keep only adc0in & adc1in 
		asm bank	_port_buff
		asm or      _port_buff,w			;store new value into buffer 
update_pa_Port:
		asm mov		w,_port_buff
		asm mov     pa_Port,w				;update cap. discharge pins
#endif

#ifdef adc0
		asm sb		_port_buff.adc0_out_pin	;check if adc0in triggered
		asm jmp		_adc0_inc_acc
		asm bank	_adc0_acc
		asm jmp		_adc0_inc_acc1			;if not, move ahead
adc0_inc_acc:
		asm bank	_adc0_acc
		asm incsz	_adc0_acc				;else, increment accumulator 
adc0_inc_acc1:
		asm inc		_adc0_acc				;and prevent overflowing 
		asm dec		_adc0_acc				;by skipping second 'INC' 
		asm mov		w,_adc0_acc				;load W from accumulator 
		asm bank	_adc0_count
		asm inc		_adc0_count				;adjust adc0in timing count 
		asm sb 		$03.2					;are we done taking reading?
		asm jmp		_adc0_inc_acc2
		asm bank	_adc0in
		asm mov		_adc0in,w				;if so, update adc0in 
adc0_inc_acc2:
		asm sb 		$03.2					; 
		asm jmp		_adc0_inc_acc_done
		asm bank	_adc0_acc
		asm clr		_adc0_acc				;if so, reset accumulator 
adc0_inc_acc_done:
#endif
 
#ifdef adc1
		asm bank	_port_buff
		asm sb		_port_buff.adc1_out_pin	;check if adc1in triggered 
		asm jmp		_adc1_inc_acc
		asm bank	_adc1_acc
		asm jmp		_adc1_inc_acc1			;if not, move ahead
adc1_inc_acc:
		asm bank	_adc1_acc
		asm incsz	_adc1_acc				;else, increment accumulator 
adc1_inc_acc1:
		asm inc		_adc1_acc				;and prevent overflowing 
		asm dec		_adc1_acc				;by skipping second 'INC' 
		asm mov		w,_adc1_acc				;load W from accumulator 
		asm bank	_adc1_count
		asm inc		_adc1_count				;adjust adc1in timing count 
		asm sb 		$03.2					;are we done taking reading? 
		asm jmp		_adc1_inc_acc2
		asm bank	_adc1in
		asm mov		_adc1in,w				;if so, update adc1in 
adc1_inc_acc2:
		asm sb 		$03.2					; 
		asm jmp		_adc1_inc_acc_done
		asm bank	_adc1_acc
		asm clr		_adc1_acc				;if so, reset accumulator 
adc1_inc_acc_done:
#endif
}
/*****************************************************************************************
 end - VP_pwm_adc()
 *****************************************************************************************/
